@@ -2,6 +2,7 @@
 #define R_STR_H
 
 #include <wchar.h>
+#include <stdarg.h>
 #include "r_str_util.h"
 #include "r_list.h"
 
@@ -21,10 +22,41 @@ typedef enum {
 
 typedef int (*RStrRangeCallback) (void *, int);
 
+typedef struct r_charset_rune_t {
+	ut8 *ch;
+	ut8 *hx;
+	struct r_charset_rune_t *left;
+	struct r_charset_rune_t *right;
+} RCharsetRune;
+
+typedef struct r_charset_t {
+	bool loaded;
+	Sdb *db;
+	Sdb *db_char_to_hex;
+	RCharsetRune *custom_charset;
+	size_t encode_maxkeylen;
+	size_t decode_maxkeylen;
+} RCharset;
+
 #define R_STR_ISEMPTY(x) (!(x) || !*(x))
 #define R_STR_ISNOTEMPTY(x) ((x) && *(x))
 #define R_STR_DUP(x) ((x) ? strdup ((x)) : NULL)
 #define r_str_array(x,y) ((y>=0 && y<(sizeof(x)/sizeof(*x)))?x[y]:"")
+R_API RCharset *r_charset_new(void);
+R_API void r_charset_free(RCharset *charset);
+R_API RCharsetRune *r_charset_rune_new(const ut8 *ch, const ut8 *hx);
+R_API void r_charset_rune_free(RCharsetRune *rcr);
+R_API size_t r_charset_encode_str(RCharset *rc, ut8 *out, size_t out_len, const ut8 *in, size_t in_len);
+R_API size_t r_charset_decode_str(RCharset *rc, ut8 *out, size_t out_len, const ut8 *in, size_t in_len);
+R_API bool r_charset_open(RCharset *c, const char *cs);
+R_API bool r_charset_use(RCharset *c, const char *cf);
+R_API RList *r_charset_list(RCharset *c);
+R_API void r_charset_close(RCharset *c);
+R_API RCharsetRune * add_rune(RCharsetRune *rcsr, const ut8 *ch, const ut8 *hx);
+R_API RCharsetRune *search_from_hex(RCharsetRune *rcsr, const ut8 *hx);
+R_API RCharsetRune *search_from_char(RCharsetRune *rcsr, const ut8 *ch);
+
+// str
 R_API char *r_str_repeat(const char *ch, int sz);
 R_API const char *r_str_pad(const char ch, int len);
 R_API const char *r_str_rstr(const char *base, const char *p);
@@ -35,30 +67,33 @@ R_API int r_str_bounds(const char *str, int *h);
 R_API char *r_str_crop(const char *str, unsigned int x, unsigned int y, unsigned int x2, unsigned int y2);
 R_API char *r_str_scale(const char *r, int w, int h);
 R_API bool r_str_range_in(const char *r, ut64 addr);
-R_API int r_str_len_utf8(const char *s);
-R_API int r_str_len_utf8_ansi(const char *str);
-R_API int r_str_len_utf8char(const char *s, int left);
-R_API int r_str_utf8_charsize(const char *str);
-R_API int r_str_utf8_charsize_prev(const char *str, int prev_len);
-R_API int r_str_utf8_charsize_last(const char *str);
+R_API int r_str_size(const char *s, int *rows);
+R_API size_t r_str_len_utf8(const char *s);
+R_API size_t r_str_len_utf8_ansi(const char *str);
+R_API size_t r_str_len_utf8char(const char *s, int left);
+R_API size_t r_str_utf8_charsize(const char *str);
+R_API size_t r_str_utf8_charsize_prev(const char *str, int prev_len);
+R_API size_t r_str_utf8_charsize_last(const char *str);
 R_API void r_str_filter_zeroline(char *str, int len);
-R_API int r_str_utf8_codepoint(const char *s, int left);
-R_API bool r_str_char_fullwidth(const char *s, int left);
+R_API size_t r_str_utf8_codepoint(const char *s, size_t left);
+R_API bool r_str_char_fullwidth(const char *s, size_t left);
 R_API int r_str_write(int fd, const char *b);
-R_API void r_str_ncpy(char *dst, const char *src, size_t n);
+R_API size_t r_str_ncpy(char *dst, const char *src, size_t n);
 R_API void r_str_sanitize(char *c);
 R_API char *r_str_sanitize_sdb_key(const char *s);
 R_API const char *r_str_casestr(const char *a, const char *b);
 R_API const char *r_str_firstbut(const char *s, char ch, const char *but);
+R_API const char *r_str_firstbut_escape(const char *s, char ch, const char *but);
 R_API const char *r_str_lastbut(const char *s, char ch, const char *but);
 R_API int r_str_split(char *str, char ch);
 R_API RList *r_str_split_list(char *str, const char *c, int n);
-R_API RList *r_str_split_duplist(const char *str, const char *c);
-R_API int *r_str_split_lines(char *str, int *count);
+R_API RList *r_str_split_duplist(const char *str, const char *c, bool trim);
+R_API size_t *r_str_split_lines(char *str, size_t *count);
 R_API char* r_str_replace(char *str, const char *key, const char *val, int g);
 R_API char *r_str_replace_icase(char *str, const char *key, const char *val, int g, int keep_case);
 R_API char *r_str_replace_in(char *str, ut32 sz, const char *key, const char *val, int g);
-#define r_str_cpy(x,y) memmove(x,y,strlen(y)+1);
+#define r_str_cpy(x,y) memmove ((x), (y), strlen (y) + 1);
+#define r_str_cat(x,y) memmove ((x) + strlen (x), (y), strlen (y) + 1);
 R_API int r_str_bits(char *strout, const ut8 *buf, int len, const char *bitz);
 R_API int r_str_bits64(char *strout, ut64 in);
 R_API ut64 r_str_bits_from_string(const char *buf, const char *bitz);
@@ -74,14 +109,16 @@ R_API int r_str_arg_unescape(char *arg);
 R_API char **r_str_argv(const char *str, int *_argc);
 R_API void r_str_argv_free(char **argv);
 R_API char *r_str_new(const char *str);
-R_API int r_snprintf (char *string, int len, const char *fmt, ...);
+R_API int r_snprintf (char *string, int len, const char *fmt, ...) R_PRINTF_CHECK(3, 4);
 R_API bool r_str_is_ascii(const char *str);
 R_API char *r_str_nextword(char *s, char ch);
-R_API int r_str_is_printable(const char *str);
-R_API int r_str_is_printable_limited(const char *str, int size);
+R_API bool r_str_is_printable(const char *str);
+R_API bool r_str_is_printable_limited(const char *str, int size);
 R_API bool r_str_is_printable_incl_newlines(const char *str);
 R_API char *r_str_appendlen(char *ptr, const char *string, int slen);
-R_API char *r_str_newf(const char *fmt, ...);
+R_API char *r_str_newf(const char *fmt, ...) R_PRINTF_CHECK(1, 2);
+R_API char *r_str_newvf(const char *fmt, va_list ap);
+R_API int r_str_distance(const char *a, const char *b);
 R_API char *r_str_newlen(const char *str, int len);
 R_API const char *r_str_sysbits(const int v);
 R_API char *r_str_trunc_ellipsis(const char *str, int len);
@@ -89,9 +126,9 @@ R_API const char *r_str_bool(int b);
 R_API bool r_str_is_true(const char *s);
 R_API bool r_str_is_false(const char *s);
 R_API bool r_str_is_bool(const char *val);
-R_API const char *r_str_ansi_chrn(const char *str, int n);
-R_API int r_str_ansi_len(const char *str);
-R_API int r_str_ansi_nlen(const char *str, int len);
+R_API const char *r_str_ansi_chrn(const char *str, size_t n);
+R_API size_t r_str_ansi_len(const char *str);
+R_API size_t r_str_ansi_nlen(const char *str, size_t len);
 R_API int r_str_ansi_trim(char *str, int str_len, int n);
 R_API int r_str_ansi_filter(char *str, char **out, int **cposs, int len);
 R_API char *r_str_ansi_crop(const char *str, unsigned int x, unsigned int y, unsigned int x2, unsigned int y2);
@@ -103,12 +140,14 @@ R_API int r_str_word_set0_stack(char *str);
 R_API const char *r_str_word_get0(const char *str, int idx);
 R_API char *r_str_word_get_first(const char *string);
 R_API void r_str_trim(char *str);
+R_API char *r_str_wrap(const char *str, int w);
 R_API char *r_str_trim_dup(const char *str);
 R_API char *r_str_trim_lines(char *str);
 R_API void r_str_trim_head(char *str);
 R_API const char *r_str_trim_head_ro(const char *str);
 R_API const char *r_str_trim_head_wp(const char *str);
 R_API void r_str_trim_tail(char *str);
+R_API void r_str_trim_args(char *str);
 R_API ut32 r_str_hash(const char *str);
 R_API ut64 r_str_hash64(const char *str);
 R_API char *r_str_trim_nc(char *str);
@@ -124,7 +163,8 @@ R_API int r_str_casecmp(const char *dst, const char *orig);
 R_API int r_str_ncasecmp(const char *dst, const char *orig, size_t n);
 R_API int r_str_ccpy(char *dst, char *orig, int ch);
 R_API const char *r_str_get(const char *str);
-R_API const char *r_str_get2(const char *str);
+R_API const char *r_str_get_fail(const char *str, const char *failstr);
+R_API const char *r_str_getf(const char *str);
 R_API char *r_str_ndup(const char *ptr, int len);
 R_API char *r_str_dup(char *ptr, const char *string);
 R_API int r_str_inject(char *begin, char *end, char *str, int maxlen);
@@ -135,6 +175,7 @@ R_API wchar_t *r_str_mb_to_wc(const char *buf);
 R_API char *r_str_wc_to_mb(const wchar_t *buf);
 R_API wchar_t *r_str_mb_to_wc_l(const char *buf, int len);
 R_API char *r_str_wc_to_mb_l(const wchar_t *buf, int len);
+R_API const char *r_str_str_xy(const char *s, const char *word, const char *prev, int *x, int *y);
 
 typedef void(*str_operation)(char *c);
 
@@ -146,7 +187,10 @@ R_API int r_str_re_replace(const char *str, const char *reg, const char *sub);
 R_API int r_str_path_unescape(char *path);
 R_API char *r_str_path_escape(const char *path);
 R_API int r_str_unescape(char *buf);
+R_API char *r_str_sanitize_r2(const char *buf);
 R_API char *r_str_escape(const char *buf);
+R_API char *r_str_escape_sh(const char *buf);
+R_API char *r_str_escape_sql(const char *buf);
 R_API char *r_str_escape_dot(const char *buf);
 R_API char *r_str_escape_latin1(const char *buf, bool show_asciidot, bool esc_bslash, bool colors);
 R_API char *r_str_escape_utf8(const char *buf, bool show_asciidot, bool esc_bslash);
@@ -163,16 +207,18 @@ R_API char *r_str_utf16_decode(const ut8 *s, int len);
 R_API int r_str_utf16_to_utf8(ut8 *dst, int len_dst, const ut8 *src, int len_src, int little_endian);
 R_API char *r_str_utf16_encode(const char *s, int len);
 R_API char *r_str_escape_utf8_for_json(const char *s, int len);
+R_API char *r_str_escape_utf8_for_json_strip(const char *buf, int buf_size);
+R_API char *r_str_encoded_json(const char *buf, int buf_size, int encoding);
 R_API char *r_str_home(const char *str);
 R_API char *r_str_r2_prefix(const char *str);
-R_API int r_str_nlen(const char *s, int n);
-R_API int r_str_nlen_w(const char *s, int n);
-R_API int r_wstr_clen(const char *s);
+R_API size_t r_str_nlen(const char *s, int n);
+R_API size_t r_str_nlen_w(const char *s, int n);
+R_API size_t r_wstr_clen(const char *s);
 R_API char *r_str_prepend(char *ptr, const char *string);
 R_API char *r_str_prefix_all(const char *s, const char *pfx);
 R_API char *r_str_append(char *ptr, const char *string);
 R_API char *r_str_append_owned(char *ptr, char *string);
-R_API char *r_str_appendf(char *ptr, const char *fmt, ...);
+R_API char *r_str_appendf(char *ptr, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
 R_API char *r_str_appendch(char *x, char y);
 R_API void r_str_case(char *str, bool up);
 R_API void r_str_trim_path(char *s);
@@ -192,6 +238,7 @@ R_API char *r_qrcode_gen(const ut8 *text, int len, bool utf8, bool inverted);
 R_API char *r_str_from_ut64(ut64 val);
 R_API void r_str_stripLine(char *str, const char *key);
 R_API char *r_str_list_join(RList *str, const char *sep);
+R_API char *r_str_array_join(const char **a, size_t n, const char *sep);
 
 R_API const char *r_str_sep(const char *base, const char *sep);
 R_API const char *r_str_rsep(const char *base, const char *p, const char *sep);
